@@ -57,11 +57,12 @@ class BaseView(object):
             based on exposed methods and actions
         """
         if self.base_permissions is None:
-            self.base_permissions = [
-                ('can_' + attr_name)
-                for attr_name in dir(self)
-                if hasattr(getattr(self, attr_name), '_urls')
-            ]
+            self.base_permissions = set()
+            for attr_name in dir(self):
+                if hasattr(getattr(self, attr_name), '_permission_name'):
+                    permission_name = getattr(getattr(self, attr_name), '_permission_name')
+                    self.base_permissions.add('can_' + permission_name)
+            self.base_permissions = list(self.base_permissions)
 
 
     def create_blueprint(self, appbuilder,
@@ -444,7 +445,7 @@ class BaseCRUDView(BaseModelView):
             if hasattr(func, '_action'):
                 action = ActionItem(*func._action, func=func)
                 self.base_permissions.append(action.name)
-                self.actions[action.name] = (action)
+                self.actions[action.name] = action
 
 
     def _init_forms(self):
